﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using TMD.MessagingSystem.DataOps;
using TMD.MessagingSystem.Helpers.Enums;
using TMD.MessagingSystem.ModelDtos;

namespace TMD.MessagingSystem.Tests.UnitTests.DataOpsTests
{
    public class DataOpsMessagesTests
    {
        [TestClass]
        public class GivenIWantToTestDataOpsMessages
        {
            private DataOpsMessages _dataOpsMessages { get; set; }

            public GivenIWantToTestDataOpsMessages()
            {
                _dataOpsMessages = new DataOpsMessages();
            }

            #region Happy Tests 
                [TestMethod]
                public void WhenICallTheGetMessagesMethodWithGoodParameters_ThenIWillValadateTheMessageResult()
                {
                    //Arrange
                    var componentName      = "MessagingSystem";
                    MessageTag messageTag  = MessageTag.NoRecordsFound;
                    var messageInfoRequest = CreateMessageInfoRequest(componentName, messageTag);

                    var messageType = "Information";
                    var message     = "No Records Found in the Messaging System Request";
                    var IsActive    = true;

                    //Act
                    var result = _dataOpsMessages.GetMessages(messageInfoRequest);

                    //Assert
                    Assert.IsNotNull(result);
                    Assert.IsTrue(result.Success);
                    Assert.IsTrue(result.ErrorList.Count == 0);

                    Assert.AreEqual(messageTag.ToString(), result.SystemMessage.MessageTag);
                    Assert.AreEqual(componentName, result.SystemMessage.ComponentName);
                    Assert.AreEqual(messageType, result.SystemMessage.MessageType);
                    Assert.AreEqual(message, result.SystemMessage.Message);
                    Assert.AreEqual(IsActive, result.SystemMessage.IsActive);
                }
            #endregion

            #region Sad Tests 
                [TestMethod]
                public void WhenICallTheGetMessagesMethodWithBadParameters_ThenIWillValadateTheErrorObjectResult()
                {
                    //Arrange
                    var componentName      = "MessagingSystem";
                    MessageTag messageTag  = MessageTag.None;
                    var messageInfoRequest = CreateMessageInfoRequest(componentName, messageTag);

                    var message = "No Message Found";

                    //Act
                    var result = _dataOpsMessages.GetMessages(messageInfoRequest);

                    //Assert
                    Assert.IsNotNull(result);
                    Assert.IsFalse(result.Success);
                    Assert.IsTrue(result.ErrorList.Count == 1);
                    Assert.AreEqual(message, result.ErrorList[0].Message);
                }
                [TestMethod]
                public void WhenICallTheGetMessagesMethodWithIsActiveFalse_ThenIWillValadateTheErrorObjectResult()
                {
                    //Arrange
                    var componentName = "MessagingSystem";
                    MessageTag messageTag = MessageTag.InactiveRecord;
                    var messageInfoRequest = CreateMessageInfoRequest(componentName, messageTag);

                    var message = "No Message Found";

                    //Act
                    var result = _dataOpsMessages.GetMessages(messageInfoRequest);

                    //Assert
                    Assert.IsNotNull(result);
                    Assert.IsFalse(result.Success);
                    Assert.IsTrue(result.ErrorList.Count == 1);
                    Assert.AreEqual(message, result.ErrorList[0].Message);
                }
            #endregion

            #region Helpers 
                private MessageInfoRequest CreateMessageInfoRequest(string componentName, MessageTag messageTag)
                {
                    return new MessageInfoRequest
                            {
                                ComponentName = componentName,
                                MessageTag    = messageTag.ToString()
                            };
                }
            #endregion
        }        
    }

}
