﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TMD.MessagingSystem.ModelDtos.Aggreagates;

namespace TMD.MessagingSystem.Helpers
{
    public abstract class ErrorHandling
    {
        public List<ErrorInfo> ErrorList { get; set; }

        protected ErrorHandling()
        {
            ErrorList = new List<ErrorInfo>();
        }

        public void AddError(Exception ex)
        {
            AddError(new ErrorInfo(ex));
        }

        public void AddError(string errorMessage)
        {
            AddError(new ErrorInfo
            {
                Message = errorMessage
            }
                );
        }

        public void BubbleErrors(IEnumerable<ErrorInfo> errorList)
        {
            var errorInfos = errorList as IList<ErrorInfo> ?? errorList.ToList();
            if (errorList != null && errorInfos.Any())
            {
                ErrorList.AddRange(errorInfos);
            }
        }

        public string GetErrorMessage()
        {
            if (ErrorList != null && ErrorList.Any())
            {
                var sb = new StringBuilder()
                    .AppendLine("The following error(s) occurred: ");

                foreach (var item in ErrorList)
                {
                    sb.AppendLine(item.Message);
                }
                return sb.ToString();
            }
            return "No Error Messages";

        }

        #region Add Error Helper
        private void AddError(ErrorInfo err)
        {
            ErrorList.Add(err);
        }
        #endregion
    }
}
